var async    = require('async');
var canvass;
module.exports = {
    '@tags': ['cf_output'],
    before: function (client) {
        client
            .url(client.launchUrl)
            .waitForElementVisible('body', 1000);
    },

    after: function (client) {

        client.end();
    },
    /**
     *  Test: 6-001
     *  Expects:
     *      - Hark Designer will be loaded in the browser.
     *      - Nodes should be drawn in the sheet
     */
    "6-001": function (client) {
        async.series([
                //load HARK Designer
                function(callback){
                    canvass = client.page.canvass();
                    canvass.navigate();

                    //check if body is now visible
                    canvass.expect.element('body').to.be.present;

                    //should have Main sheet
                    canvass.expect.element('@mainSheet').to.be.visible;

                    callback();
                },
                //add nodeLocalizeMUSIC
                function (callback) {
                    canvass.addNodeByName('LocalizeMUSIC', function () {
                        callback();
                    });
                }
            ],
            function(){
                //finished
            });
    },
    /**
     *  Test: 6-002
     *  Expects:
     *      - Context menu will be displayed
     */
    "6-002": function (client) {
        async.series([
                //mouse over nodeLocalizeMUSIC
                function (callback) {
                    client.moveToElement('#node_LocalizeMUSIC_1', 0, 0, function () {
                        callback();
                    });
                },
                //right click
                function (callback) {
                    client.mouseButtonClick('right', function(){
                        callback();
                    });
                },
                //context menu should be displayed
                function (callback) {
                    canvass.waitForElementPresent('@mainContextMenu', 1000, function () {
                        canvass.expect.element('@mainContextMenu').to.be.visible;
                        callback();
                    });
                }
            ],
            function(){
                //finished
            });
    },
    /**
     *  Test: 6-003
     *  Expects:
     *      - "Add Output" dialog will be displayed.
     */
    "6-003": function (client) {
        async.series([
                //Add Output option should be displayed
                function (callback) {
                    canvass.waitForElementPresent('@addOutputOption', 1000, function () {
                        callback();
                    });
                },
                //click Add Output option
                function (callback) {
                    canvass.click('@addOutputOption', function() {
                        callback();
                    });
                },
                //Add Input Dialog should be displayed
                function (callback) {
                    canvass.waitForElementVisible('@addInputOutputDialog', 1000, function () {
                        callback();
                    });
                }
            ],
            function(){
                //finished
            });
    },
    /**
     *  Test: 6-004
     *  Expects:
     *      - Dialog should be close after the "Add" button is clicked.
     *      - Node element should be added an input in the right side of the element with a name of "OUTPUT_1".
     */
    "6-004": function (client) {
        async.series([
                //Add button should be displayed
                function (callback) {
                    canvass.waitForElementVisible('@addInputOutputButton', 1000, function () {
                        callback();
                    });
                },
                //clear Output name text
                function (callback) {
                    canvass.clearValue('@addInputOutputText', function () {
                        callback();
                    });
                },
                //Set Output name
                function (callback) {
                    canvass.setValue('@addInputOutputText', 'OUTPUT_1', function () {
                        callback();
                    });
                },
                //click Add button
                function (callback) {
                    canvass.click('@addInputOutputButton', function(){
                        callback();
                    });
                },
                //Add Output dialog should be closed
                function (callback) {
                    canvass.waitForElementNotVisible('@addInputOutputDialog', 1000, function () {
                        callback();
                    });
                },
                //OUTPUT_1 should be displayed
                function (callback){
                    canvass.expect.element('@nodeLocalizeMUSICOutput1').to.be.present;
                    callback();
                }
            ],
            function(){
                //finished
            });
    },
    /**
     *  Test: 6-005
     *  Expects:
     *      - Dialog should be close after the "Add" button is clicked.
     *      - Node element should be added an input in the right side of the element with a name of "OUTPUT_2".
     */
    "6-005": function (client) {
        async.series([
                //add OUTPUT_2 input to nodeLocalizeMUSIC1
                function (callback) {
                    canvass.addOutput(client, 'OUTPUT_2', '#node_LocalizeMUSIC_1', function () {
                        callback();
                    });
                },
                //OUTPUT_2 should be displayed
                function (callback){
                    canvass.expect.element('@nodeLocalizeMUSICOutput2').to.be.present;
                    callback();
                }
            ],
            function(){
                //finished
            });
    },
    /**
     *  Test: 6-006
     *  Expects:
     *      - Context menu will be displayed. Items for the context menu:
     *          1. Set as Input
     *          2. Delete this Input
     *      - For the default input of the node, items for the context menu:
     *          1. Set as Input
     *              only.
     */
    "6-006": function (client) {
        async.series([
                //mouse over OUTPUT_2
                function (callback) {
                    client
                        .moveToElement('#node_LocalizeMUSIC_1_delim_OUTPUT_2', 3, 3, function () {
                            callback();
                        });
                },
                //right click
                function (callback) {
                    client.mouseButtonClick('right', function(){
                        callback();
                    })
                },
                //context meny should be displayed
                function (callback) {
                    canvass.waitForElementPresent('@mainContextMenu2', 1000, function(){
                        canvass.expect.element('@mainContextMenu2').to.be.present;
                        callback();
                    });
                },
                //Delete this Output and Set as Output options should be displayed
                function (callback) {
                    canvass.expect.element('@deleteIOOption').to.be.present;
                    canvass.expect.element('@setAsIOOption').to.be.present;
                    callback();
                },
            ],
            function(){
                //finished
            });
    },
    /**
     *  Test: 6-007
     *  Expects:
     *      - OUTPUT_2 should be deleted.
     */
    "6-007": function (client) {
        async.series([
                //click Delete Output option
                function (callback) {
                    canvass.click('@deleteIOOption', function() {
                        callback();
                    });
                },
                //OUTPUT_2 should be deleted
                function (callback) {
                    canvass.expect.element('@nodeLocalizeMUSICOutput2').to.not.be.present;
                    callback();
                }
            ],
            function(){
                //finished
            });
    },
    /**
     *  Test: 6-008
     *  Expects:
     *      - Context menu will be displayed.
     *      - Dialog will appper with default value depends on the name of the output. In this case, "OUTPUT" is the set value.
     *      - Red labeled "OUTPUT" should be drawn on the left side of the node.
     */
    "6-008": function (client) {
        async.series([
                //set OUTPUT of nodeLocalizeMusic1 as OUTPUT
                function (callback) {
                    canvass.setAsIO(client, '#node_LocalizeMUSIC_1_delim_OUTPUT', 'OUTPUT', function () {
                        callback();
                    });
                },
                //OUTPUT should have role as OUTPUT
                function (callback) {
                    canvass.expect.element('@outputRole').text.to.equal('OUTPUT');
                    callback();
                }
            ],
            function(){
                //finished
            });
    },
    /**
     *  Test: 6-009
     *  Expects:
     *      - Context menu will be displayed.
     *      - Dialog will appper with default value depends on the name of the output. In this case, "OUTPUT_1" is the set value.
     *      - Red labeled "OUTPUT_1" should be drawn on the left side of the node.
     */
    "6-009": function (client) {
        async.series([
                //set OUTPUT_1 of nodeLocalizeMusic1 as OUTPUT_1
                function (callback) {
                    canvass.setAsIO(client, '#node_LocalizeMUSIC_1_delim_OUTPUT_1', 'OUTPUT_1', function () {
                        callback();
                    });
                },
                //OUTPUT_1 should have role as OUTPUT_1
                function (callback) {
                    canvass.expect.element('@output1Role').text.to.equal('OUTPUT_1');
                    callback();
                }
            ],
            function(){
                //finished
            });
    },
    /**
     *  Test: 6-010
     *  Expects:
     *      - Save file dialog will be closed
     */
    "6-010": function (client) {
        async.series([
                //save network
                function (callback) {
                    canvass.saveNetwork(client, 'sample_output.n', function(){
                        callback();
                    });
                },
                //save sheet dialog should be closed
                function (callback){
                    canvass.expect.element('@saveSheetDialog').to.not.be.visible;
                }
            ],
            function(){
                //finished
            });
    },
    /**
     *  Test: 6-011
     *  Expects:
     *      - The saved network "sample_output.n" will be loaded in the Hark Designer.
     *      - LocalizedMUSIC with red labeled inputs should be drawn in the sheet.
     */
    "6-011": function (client) {
        async.series([
                //load sample_output.n
                function (callback) {
                    canvass.loadNetwork(client, 'sample_output.n', function () {
                        canvass.expect.element('@fileDialog').to.not.be.visible;
                        callback();
                    })
                },
                //Inputs and roles should be displayed
                function (callback) {
                    canvass.expect.element('@nodeLocalizeMUSICOutput1').to.be.present;
                    canvass.expect.element('@outputRole').to.be.present;
                    canvass.expect.element('@output1Role').to.be.present;
                    canvass.expect.element('@outputRole').text.to.equal('OUTPUT');
                    canvass.expect.element('@output1Role').text.to.equal('OUTPUT_1');
                    callback();
                }
            ],
            function(){
                //finished
            });
    },
    /**
     *  Test: 6-012
     *  Expects:
     *      - Red labeled output name will be removed.
     */
    "6-012": function (client) {
        async.series([
                //delete OUTPUT_1 role
                function (callback) {
                    canvass.deleteIO(client, '@deleteIOOption2', '#node_LocalizeMUSIC_1_delim_OUTPUT_1', function () {
                        callback();
                    });
                },
                //OUTPUT_1 role should be deleted
                function (callback) {
                    canvass.expect.element('@output1Role').text.to.not.equal('OUTPUT_1');
                    callback();
                }
            ],
            function(){
                //finished
            });
    },
    /**
     *  Test: 6-013
     *  Expects:
     *      - OUTPUT_1 will be removed.
     */
    "6-013": function (client) {
        async.series([
                //delete OUTPUT_1 role
                function (callback) {
                    canvass.deleteIO(client, '@deleteIOOption', '#node_LocalizeMUSIC_1_delim_OUTPUT_1', function () {
                        callback();
                    });
                },
                //OUTPUT_1 role should be deleted
                function (callback) {
                    canvass.expect.element('@nodeLocalizeMUSICOutput1').to.not.be.present;
                    callback();
                }
            ],
            function(){
                //finished
            });
    },
    /**
     *  Test: 6-014
     *  Expects:
     *      - Save file dialog will be closed
     */
    "6-014": function (client) {
        async.series([
                //save network
                function (callback) {
                    canvass.saveNetwork(client, 'sample_output.n', function(){
                        callback();
                    });
                },
                //save file dialog should be closed
                function (callback){
                    canvass.expect.element('@saveSheetDialog').to.not.be.visible;
                }
            ],
            function(){
                //finished
            });
    },
    /**
     *  Test: 6-015
     *  Expects:
     *      - The saved network "sample_output.n" will be loaded in the Hark Designer.
     *      - LocalizedMUSIC with red labeled outputs should be drawn in the sheet.
     */
    "6-015": function (client) {
        async.series([
                //load sample_output.n
                function (callback) {
                    canvass.loadNetwork(client, 'sample_output.n', function () {
                        canvass.expect.element('@fileDialog').to.not.be.visible;
                        callback();
                    })
                },
                //Inputs and roles should be displayed
                function (callback) {
                    canvass.expect.element('@output1Role').to.not.be.present;
                    canvass.expect.element('@nodeLocalizeMUSICOutput1').to.not.be.present;
                    canvass.expect.element('@outputRole').to.be.present;
                    canvass.expect.element('@outputRole').text.to.equal('OUTPUT');
                    callback();
                }
            ],
            function(){
                //finished
            });
    },
    /**
     *  Test: 6-016
     *  Expects:
     *      - Dialog box will appear.
     *      -it should have 1 input box for Name
     *      -It should have Add and Cancel button
     */
    "6-016": function (client) {
        async.series([
                //click + button
                function (callback) {
                    canvass.click('@addSheetButton', function(){
                        callback();
                    });
                },
                //should have input box for name
                function (callback) {
                    canvass.expect.element('@sheetName').to.be.present;
                    callback();
                },
                //should have Add and Cancel button
                function (callback) {
                    canvass.expect.element('@addSheetSubmitButton').to.be.present;
                    canvass.expect.element('@addSheetCancelButton').to.be.present;
                    callback();
                }
            ],
            function(){
                //finished
            });
    },
    /**
     *  Test: 6-017
     *  Expects:
     *      - Dialog box will be closed
     *      - New sheet with "Dynamic1" as title will be added and set as active- Dialog box will appear.
     */
    "6-017": function (client) {
        async.series([
                //set sheet name
                function (callback) {
                    canvass.setValue('@sheetName', 'Dynamic1', function(){
                        callback();
                    });
                },
                //click Add button
                function (callback) {
                    canvass.click('@addSheetSubmitButton', function(){
                        callback();
                    });
                },
                //dialog box should be closed
                function (callback) {
                    canvass.waitForElementNotVisible('@addSheetDialog', 1000, function(){
                        canvass.expect.element('@addSheetDialog').to.not.be.visible;
                        callback();

                    });
                },
                //Dynamic1 sheet should be created
                function (callback) {
                    canvass.expect.element('@dynamic1Sheet').to.be.visible;
                    callback();
                }
            ],
            function(){
                //finished
            });
    },
    /**
     *  Test: 6-018
     *  Expects:
     *      - LocalizeMusic Node should be present in the sheet.
     */
    "6-018": function (client) {
        async.series([
                //Dynamic1 sheet should be visible
                function (callback) {
                    canvass.selectSheet('@dynamic1SheetTab', function () {
                        canvass.expect.element('@dynamic1Sheet').to.be.visible;
                        callback();
                    });
                },
                //add LocalizeMUSIC node
                function (callback) {
                    canvass.addNodeByName('LocalizeMUSIC', function () {
                        callback();
                    });
                },
                //LocalizeMUSIC node should be added
                function (callback) {
                    canvass.expect.element('@nodeLocalizeMUSIC2').to.be.visible;
                    callback();

                }
            ],
            function(){
                //finished
            });
    },
    /**
     *  Test: 6-019
     *  Expects:
     *      - Context menu will be displayed.
     *      - Dialog will appper with default value depends on the name of the output. In this case, "OUTPUT" is the set value.
     *      - Red labeled "OUTPUT" should be drawn on the left side of the node.
     */
    "6-019": function (client) {
        async.series([
                //set OUTPUT of nodeLocalizeMusic2 as OUTPUT
                function (callback) {
                    canvass.setAsIO(client, '#node_LocalizeMUSIC_2_delim_OUTPUT', 'OUTPUT', function () {
                        callback();
                    });
                },
                //nodeLocalizeMUSIC2 should have OUTPUT
                function (callback) {
                    canvass.expect.element('@outputRole2').text.to.equal('OUTPUT');
                    callback();
                }
            ],
            function(){
                //finished
            });
    },
    /**
     *  Test: 6-020
     *  Expects:
     *      - Dialog box will be closed
     *      - New sheet with "Dynamic2" as title will be added and set as active
     */
    "6-020": function (client) {
        async.series([
                function (callback) {
                    //add Dynamic2 sheet
                    canvass.addSheet('Dynamic2', function () {
                        //Dynamic2 sheet should be created
                        canvass.expect.element('@dynamic2Sheet').to.be.present;
                        callback();
                    })
                }
            ],
            function(){
                //finished
            });
    },
    /**
     *  Test: 6-021
     *  Expects:
     *      - LocalizeMusic Node should be present in the sheet.
     */
    "6-021": function (client) {
        async.series([
                //Dynamic2 sheet should be visible
                function (callback) {
                    canvass.selectSheet('@dynamic2SheetTab', function () {
                        canvass.expect.element('@dynamic2Sheet').to.be.visible;
                        callback();
                    });
                },
                //add LocalizeMUSIC node
                function (callback) {
                    canvass.addNodeByName('LocalizeMUSIC', function () {
                        callback();
                    });
                },
                //LocalizeMUSIC node should be added
                function (callback) {
                    canvass.expect.element('@nodeLocalizeMUSIC3').to.be.visible;
                    callback();

                }
            ],
            function(){
                //finished
            });
    },
    /**
     *  Test: 6-022
     *  Expects:
     *      - Context menu will be displayed.
     *      - Dialog will appper with default value depends on the name of the utput. In this case, "OUTPUT" is the set value.
     *      - Red labeled "OUTPUT" should be drawn on the left side of the node.
     */
    "6-022": function (client) {
        async.series([
                //set OUTPUT of nodeLocalizeMusic3 as OUTPUT
                function (callback) {
                    canvass.setAsIO(client, '#node_LocalizeMUSIC_3_delim_OUTPUT', 'OUTPUT', function () {
                        callback();
                    });
                },
                //nodeLocalizeMUSIC3 should have OUTPUT
                function (callback) {
                    canvass.expect.element('@outputRole3').text.to.equal('OUTPUT');
                    callback();
                }
            ],
            function(){
                //finished
            });
    },
    /**
     *  Test: 6-023
     *  Expects:
     *      - Dynamic1 sheet should be displayed with one LocalizeMUSIC node and one Dynamic2 node.
     *      - Dynamic2 node should have an OUTPUT.
     */
    "6-023": function (client) {
        async.series([
                //select Dynamic1 sheet
                function (callback) {
                    canvass.selectSheet('@dynamic1SheetTab', function () {
                        //Dynamic1 sheet should be visible
                        canvass.expect.element('@dynamic1Sheet').to.be.visible;
                        callback();
                    });
                },
                //add Dynamic2 dynamic node
                function (callback) {
                    canvass.addNodeByName('Dynamic2', function () {
                        //Dynamic1 node should be added
                        canvass.expect.element('@dynamic2Node').to.be.visible;
                        //Dynamic1 node should have OUTPUT
                        canvass.expect.element('@dynamic2NodeOutput').to.be.present;

                        callback();
                    });
                }
            ],
            function(){
                //finished
            });
    },
    /**
     *  Test: 6-024
     *  Expects:
     *      - OUTPUT of Dynamic2 node should be connected to INPUT of LocalizeMUSIC node.
     */
    "6-024": function (client) {
        async.series([
                function (callback) {
                    //reposition nodeLocalizeMusic2
                    canvass.repositionNode(client, '#node_LocalizeMUSIC_2', 500, 200, function () {
                        callback();
                    });
                },
                function (callback) {
                    //link nodeLocalizeMUSIC2 OUTPUT to nodeDynamic2 INPUT
                    canvass.linkNodes(client, '#node_Dynamic2_1_delim_OUTPUT > div', '#node_LocalizeMUSIC_2_delim_INPUT > div', 50, 0, function () {
                        callback();
                    });
                }
            ],
            function(){
                //finished
            });
    },
    /**
     *  Test: 6-025
     *  Expects:
     *      - MAIN sheet should be displayed with one LocalizeMUSIC node and one Dynamic1 node.
     *      - Dynamic1 node should have an OUTPUT.
     */
    "6-025": function (client) {
        async.series([
                //select MAIN sheet
                function (callback) {
                    canvass.selectSheet('@mainSheetTab', function () {
                        //MAIN sheet should be visible
                        canvass.expect.element('@mainSheet').to.be.visible;
                        callback();
                    });
                },
                //add Dynamic1 dynamic node
                function (callback) {
                    canvass.addNodeByName('Dynamic1', function () {
                        //Dynamic1 node should be added
                        canvass.expect.element('@dynamic1Node').to.be.visible;
                        //Dynamic1 node should have OUTPUT
                        canvass.expect.element('@dynamic1NodeOutput').to.be.present;

                        callback();
                    });
                }
            ],
            function(){
                //finished
            });
    },
    /**
     *  Test: 6-026
     *  Expects:
     *      - OUTPUT of Dynamic1 node should be connected to INPUT of LocalizeMUSIC node
     */
    "6-026": function (client) {
        async.series([
                function (callback) {
                    //reposition nodeDynamic1
                    canvass.repositionNode(client, '#node_LocalizeMUSIC_1', 500, 200, function () {
                        callback();
                    });
                },
                function (callback) {
                    //link nodeLocalizeMUSIC1 OUTPUT to nodeDynamic1 INPUT
                    canvass.linkNodes(client, '#node_Dynamic1_1_delim_OUTPUT > div', '#node_LocalizeMUSIC_1_delim_INPUT > div', 50, 0, function () {
                        callback();
                    });
                },
            ],
            function(){
                //finished
            });
    },
    /**
     *  Test: 6-027
     *  Expects:
     *      - OUTPUT will be removed
     */
    "6-027": function (client) {
        async.series([
                function (callback) {
                    //go to Dynamic2 sheet
                    canvass.selectSheet('@dynamic2SheetTab', function () {
                        //Dynamic1 sheet should be visible
                        canvass.expect.element('@dynamic2Sheet').to.be.visible;
                        callback();
                    });
                },
                function (callback) {
                    //delete nodeDynamic2 INPUT
                    canvass.deleteIO(client, '@dynamic2SheetDeleteIOOption', '#node_LocalizeMUSIC_3_delim_OUTPUT', function () {
                        //role must be removed
                        canvass.expect.element('@outputRole3').text.to.not.equal('OUTPUT');
                        callback();
                    });
                }
            ],
            function(){
                //finished
            });
    },
    /**
     *  Test: 6-028
     *  Expects:
     *      - INPUT of LocalizeMUSIC node will be disconnected.
     *      - Dynamic2 should not have any output.
     */
    "6-028": function (client) {
        async.series([
                function (callback) {
                    //go to Dynamic1 sheet
                    canvass.selectSheet('@dynamic1SheetTab', function () {
                        //MAIN sheet should be visible
                        canvass.expect.element('@dynamic1Sheet').to.be.visible;
                        //nodeDynamic1 should have no OUTPUT
                        canvass.expect.element('@dynamic2NodeOutput').to.not.be.present;

                        callback();
                    });
                }
            ],
            function(){
                //finished
            });
    },
    /**
     *  Test: 6-029
     *  Expects:
     *      - OUTPUT will be removed
     */
    "6-029": function (client) {
        async.series([
                function (callback) {
                    //delete nodeDynamic1 INPUT
                    canvass.deleteIO(client, '@dynamic1SheetDeleteIOOption', '#node_LocalizeMUSIC_2_delim_OUTPUT', function () {
                        //role must be removed
                        canvass.expect.element('@outputRole3').text.not.to.equal('OUTPUT');
                        callback();
                    });
                }
            ],
            function(){
                //finished
            });
    },
    /**
     *  Test: 6-030
     *  Expects:
     *      - INPUT of LocalizeMUSIC node will be disconnected.
     *      -Dynamic1 should not have any output.
     */
    "6-030": function (client) {
        async.series([
                function (callback) {
                    //go to MAIN sheet
                    canvass.selectSheet('@mainSheetTab', function () {
                        //MAIN sheet should be visible
                        canvass.expect.element('@mainSheet').to.be.visible;
                        //nodeDynamic1 should have no OUTPUT
                        canvass.expect.element('@dynamic1NodeOutput').to.not.be.present;

                        callback();
                    });
                }
            ],
            function(){
                //finished
            });
    }
};