var async    = require('async');
var canvass;
module.exports = {
    '@tags': ['cf_property'],
    before: function (client) {
        client
            .url(client.launchUrl)
            .waitForElementVisible('body', 1000);
    },

    after: function (client) {

        client.end();
    },
    /**
     *  Test: 7-001
     *  Expects:
     *      - Hark Designer will be loaded in the browser.
     *      - Node should be drawn in the sheet.
     */
    "7-001": function (client) {
        async.series([
                //load HARK Designer
                function(callback){
                    canvass = client.page.canvass();
                    canvass.navigate();

                    //check if body is now visible
                    canvass.expect.element('body').to.be.present;

                    //should have Main sheet
                    canvass.expect.element('@mainSheet').to.be.visible;

                    callback();
                },
                //add LocalizeMUSIC node
                function (callback) {
                    canvass.addNodeByName('LocalizeMUSIC', function () {
                        callback();
                    });
                },
                //check if nodeLocalizeMUSIC1 is added
                function(callback) {
                    canvass.expect.element('@nodeLocalizeMUSIC1').to.be.visible;
                    callback();
                }
            ],
            function(){
                //finished
            });
    },
    /**
     *  Test: 7-002
     *  Expects:
     *      - Context menu will appear
     */
    "7-002": function (client) {
        async.series([
                //mouse over nodeLocalizeMUSIC1
                function (callback) {
                    client.moveToElement('#node_LocalizeMUSIC_1', 0, 0, function () {
                        callback();
                    });
                },
                //right click
                function (callback) {
                    client.mouseButtonClick('right', function(){
                        callback();
                    });
                },
                function (callback) {
                    canvass.waitForElementPresent('@mainContextMenu', 1000, function () {
                        canvass.expect.element('@mainContextMenu').to.be.visible;
                        callback();
                    });
                }
            ],
            function(){
                //finished
            });
    },
    /**
     *  Test: 7-003
     *  Expects:
     *      - "Parameters of node" dialog will appear.
     */
    "7-003": function (client) {
        async.series([
                //there should be Property option
                function (callback) {
                    canvass.waitForElementPresent('@propertyOption', 1000, function () {
                        callback();
                    })
                },
                //click Help option
                function (callback) {
                    canvass.click('@propertyOption', function(){
                        callback();
                    });
                },
                //Parameters of node dialog should appear
                function (callback) {
                    canvass.waitForElementPresent('@nodeLocalizeMusic1PropertyDialog', 1000, function () {
                        canvass.expect.element('@nodeLocalizeMusic1PropertyDialog').to.be.visible;
                        callback();
                    });
                }
            ],
            function(){
                //finished
            });
    },
    /**
     *  Test: 7-004
     *  Expects:
     *      - "Parameters of node" dialog will close.
     */
    "7-004": function (client) {
        async.series([
                //clear LENGTH property
                function (callback) {
                    canvass.clearValue('@nodeLocalizeMusic1LENGTH', function () {
                        callback();
                    });
                },
                //set LENGTH to 1024
                function (callback) {
                    canvass.setValue('@nodeLocalizeMusic1LENGTH', '1024', function () {
                        callback();
                    });
                },
                //click ok
                function (callback) {
                    canvass.closeProperty('@nodeLocalizeMusic1PropertyDialogOkButton', function () {
                        canvass.api.pause(1000, function () {
                            callback();
                        });
                    })
                },
                //Parameters of node dialog should be closed
                function (callback) {
                    canvass.waitForElementNotVisible('@nodeLocalizeMusic1PropertyDialog', 1000, function () {
                        canvass.expect.element('@nodeLocalizeMusic1PropertyDialog').to.not.be.visible;
                        callback();
                    });
                }
            ],
            function(){
                //finished
            });
    },
    /**
     *  Test: 7-005
     *  Expects:
     *      - Save file dialog will be closed
     */
    "7-005": function (client) {
        async.series([
                //save network
                function (callback) {
                    canvass.saveNetwork(client, 'sample_property.n', function(){
                        callback();
                    });
                },
                //save sheet dialog should be closed
                function (callback){
                    canvass.expect.element('@saveSheetDialog').to.not.be.visible;
                }
            ],
            function(){
                //finished
            });
    },
    /**
     *  Test: 7-006
     *  Expects:
     *      - The saved network "sample_property.n" will be loaded in the Hark Designer.
     *      - Length parameter of  LocalizedMUSIC should have the value of 1024.
     */
    "7-006": function (client) {
        async.series([
                //load sample_property.n
                function (callback) {
                    canvass.loadNetwork(client, 'sample_property.n', function () {
                        canvass.expect.element('@fileDialog').to.not.be.visible;
                        callback();
                    })
                },
                //open property dialog
                function (callback) {
                    canvass.openProperty(client, '#node_LocalizeMUSIC_1', '@nodeLocalizeMusic1LENGTH', '@nodeLocalizeMusic1PropertyDialog', function () {
                        //check if length property is 1024
                        canvass.expect.element('@nodeLocalizeMusic1LENGTH').to.have.value.that.equals('1024');
                        callback();
                    });
                },
                //close property dialog
                function (callback) {
                    canvass.closeProperty('@nodeLocalizeMusic1PropertyDialogOkButton', function () {
                        canvass.expect.element('@nodeLocalizeMusic1PropertyDialog').to.not.be.visible;
                        callback();
                    });
                }
            ],
            function(){
                //finished
            });
    }
};
