var path    = require('path')
  , fs      = require('fs')
  , util    = require('util')
  , xml2js  = require('xml2js')
  , _       = require('underscore')
  , date   = require('date-utils')
  , os     = require('os');
var multer = require('multer');
var isMultiuser = module.parent.exports.set("isMultiuser");

var exec = require('child_process').exec;
var execSync = require('child_process').execSync;

var io = null; // initiate socket.io server
var _socket = null;
var uploadqueue = [];
var distribution = os.type();

// distribution decision
if(distribution.indexOf("Linux") !== -1)   distribution = "Linux";
if(distribution.indexOf("Windows") !== -1) distribution = "Windows";
if(distribution.indexOf("Darwin") !== -1)  distribution = "Darwin";

exports.datapath = function(req, res){
    //var reg = new RegExp(path.delimiter, 'g');
    //var url = "file://" + process.cwd().replace(reg, "/") + "/" + getUserdirPath(req, isMultiuser);

    // Sets root directory
    var url = fileSys.getDefaultDir(req);
    console.log(url);
    fileSys.setRoot(url);
    res.status(200).send(url);
};

exports.getFile = function(req, res){
    var username = (isMultiuser) ? req.session.user:"singleuser";
    var filename = decodeURIComponent(req.params.name);
    var root = fileSys.getRoot();
    var filepath = "";
    filepath = root + '/' + filename;

    // Comments this since querying file in database is disregarded
    /*db = new sqlite3.Database('db.userfile');
    db.serialize(function(){
        var query = "select * from files where username=(?) and filename=(?) and category=(?) and path=(?)";
        db.get(query, [username, filename, "network", filepath], function(err, row){
            if(!err && !_.isUndefined(row)){*/
                // Load network files regardless if found in database or not
                if (fs.existsSync(filepath)) {
                    var xml = fs.readFileSync(filepath, {encoding: "utf-8"});
                    xml = xml.replace(/#!.*batchflow[^<]/, "");

                    var parser = new xml2js.Parser();
                    parser.parseString(xml, function(err, result){
                        console.log(err, result);
                        if (err) {
                            res.status(400).send("Parsing XML failed: " + err);
                            console.log(err);
                        } else {
                            var object = (_.has(result.Document.$, "category")) ? xml2data(result) : xml2dataPerSheet(result);
                            var data = {
                                type: "network",
                                name: filename,
                                object: object
                            };
                            res.status(200).send(data);
                        }
                    });
                }
    /*        } else{
                res.send(404, "File not found");
            }
        });
    });
    db.close();*/
};

exports.dlFile = function(req, res){
    var username = (isMultiuser)?req.session.user:"singleuser";
    var filename = decodeURIComponent(req.params.name);
    //var filetype = decodeURIComponent(req.params.type);
    var root = fileSys.getRoot();
    var filepath = root + '/' + filename;

    if(fs.existsSync(filepath)){
        res.download(filepath);
    } else {
        res.status(404).send("File not found");
    }
};

// Comments this since native method is used to determine file or directory
/*var isNotDir = function(filepath){return !isDir(filepath);};
var isDir = function( filepath ) {
    if( /\/$/.test( filepath ) ) {
        filepath = filepath.substr( 0, filepath.lastIndexOf('/')-1 );
    }
    return (fs.existsSync( filepath ) && fs.statSync( filepath ).isDirectory());
};*/
exports.getAllFiles = function(req, res, next){
    if(isMultiuser && _.isUndefined(req.session.user)){
        console.log("redirect from getAllFiles: isMultiuser=",
            isMultiuser, ", req.session.user=", req.session.user);
        res.redirect("/login");
    }
    var username = (isMultiuser)?req.session.user:"singleuser";
    var root = (fileSys.getRoot() === '' || _.isUndefined(fileSys.getRoot())) ? fileSys.getDefaultDir():fileSys.getRoot();
    var result = [];

    fs.readdir(root, function (error, data) {
        // If directory is not present, send empty result
        if (error) {
            res.status(404).send(error);
            return next(error);
        } else {
            // Reads file info from current directory
            _.each(data, function(f){
                var filePath = root + '/' + f;
                var stats = fs.statSync(filePath);
                if (stats.isFile()) {
                    var fileInfo = fileCache.getData(root, f, stats.mtime);
                    if (fileInfo && fileInfo.type) {
                        // file info exist within fileCache
                        result.push({"name": f, "type": fileInfo.type});
                    } else {
                        // file info don't exist within fileCache
                        var type = checkType(f); //determine what type of file

                        if (type === 'zip') {
                            try{
                                execSync('harktoolcli-validtf ' + '"' + filePath + '"');
                                type = 'tf';
                            } catch (e) {
                                type = 'others';
                            }
                        }

                        fileCache.setData(root, f, {"filename": f, "type": type, "size": stats.size || 0, "path": filePath});
                        result.push({"name": f, "type": type});
                    }
                }
            });
 
            // Sorts results
            result = _.sortBy(result, function(r) {
                return r.type + r.name;
            });

            res.status(200).send(result);
            // delete cache of non-existing file
            _.difference(fileCache.listCache(root), _.pluck(result, "name")).forEach(function (file) {
                fileCache.deleteData(root, file);
            });
        }
    });
};

// Backend for file tree
exports.getDir = function (req,res) {
    var id = decodeURIComponent(req.query.id);
    var dir = decodeURIComponent(req.query.dir);

    var getNode = function (node) {
        if (!_.isUndefined(node) && (node !== 'undefined') && (node !== '#')) {
            return node;
        }
        else {
            return '/';
        }
    };

    switch (req.query.operation) {
        case 'get_node': // Gets directory listing
            var node = getNode(id);
            fileSys.list(dir, node, (!_.isUndefined(id)) && (id === '#'), function (result) {
                if (result.err) {
                    res.status(404).send(result.err);
                }
                else {
                    res.status(200).send(result);
                }
            });
            break;
    }
};

exports.setDir = function (req,res) {
    var dir;
    if (req.body.dir) {
        dir = req.body.dir.trim();
    } else {
        console.log("No parameter [dir]");
        res.status(400).send("No parameter [dir]");
        return;
    }

    dir = fileSys.resolve(dir); // for removing tailing sep
    var stats;
    try {
        stats = fs.statSync(dir);
    } catch (err) {
        // No such file or directory
        console.log(err.code);
        res.status(404).send(err.code);
        return;
    }
    if (stats.isDirectory()) {
        fileSys.setRoot(dir);
        res.status(200).send({ root: fileSys.getRoot() });
    } else {
        // Not directory
        console.log("Not directory: " + dir);
        res.status(404).send("Not directory");
    }
};

exports.deleteFile = function(req, res){
    var username = (isMultiuser)?req.session.user:"singleuser";
    var filename = decodeURIComponent(req.params.name);
    var root = fileSys.getRoot();
    var filepath = "";
    var query = "";

    filepath = root + '/' + filename;
    if(fs.existsSync(filepath)) {
        fs.unlinkSync(filepath); // Deletes physical file
    }
    res.status(200).send("file deleted");
    fileCache.deleteData(root, filename);
};

exports.saveNetwork = function (req, res, next) {
    var username = (isMultiuser)?req.session.user:"singleuser";
    var filename = decodeURIComponent(req.params.name);
    var type = "network";
    var filepath = "";
    var root = fileSys.getRoot();

    if(_.isUndefined(req.body.sheets)){
        res.status(400).send("Save error: network does not have sheets.");
        return;
    }
    if(!_.contains(_.keys(req.body.sheets), "MAIN")){
        res.status(400).send("Save error: network does not have MAIN sheet.");
        return;
    }

    // Saves file with directory
    filepath = root + '/' + filename;

    saveNetwork(filepath, data2xml(req.body), req);
    res.sendStatus(200);
    fileCache.setData(root, filename, {"filename": filename, "type": type, "path": filepath});
};

exports.saveNetworkPerSheet = function (req, res, next) {
    var username = (isMultiuser)?req.session.user:"singleuser";
    var filename = req.params.name;
    var type = "network";

    if(_.isUndefined(req.body.sheets)){
        res.status(400).send("Save error: network does not have sheets.");
        return;
    }
    if(!_.contains(_.keys(req.body.sheets), "MAIN")){
        res.status(400).send("Save error: network does not have MAIN sheet.");
        return;
    }

    saveNetworkPerSheet(req.params.name, data2xmlPerSheet(req.body), req);
    res.sendStatus(200);
};

exports.getFilenames = function(req, res) {
    var dirpath = process.env.FLOWDESIGNER_PATH + "/user-generated/";
    if(!fs.existsSync(dirpath)) {
        fs.mkdirSync(dirpath);
        var def = '<?xml version="1.0"?> \n<Definitions> \n</Definitions>';
        fs.writeFileSync(dirpath + 'user-generated.def', def);
        fs.chmodSync(dirpath + 'user-generated.def', 0755);
    }
    var data = fs.readdirSync(dirpath);
    res.status(200).send(data);
};

exports.addPackage = function(req, res, next){
    var username = (isMultiuser)?req.session.user:"singleuser";
    var filename = req.params.name;
    var dirpath = module.parent.exports.set("dirname")+ "/" + getUserdirPath(req, isMultiuser)
                  + "_package/";

    console.log("filename:", filename);
    if(filename.slice(-4) !== ".def" && filename.slice(-4) !== ".tlb") {
        res.status(400).send("Extensions must be .def or .tlb");
        return;
    }

    if(fs.existsSync(dirpath + filename)) fs.unlinkSync(dirpath + filename);

/*    var options = {
        tmpDir: './tmp',
        uploadDir: path,
        uploadUrl: "/package/" + filename,
        storage : {
            type : 'local'
        },
        copyImgAsThumb : false
    };
    var uploader = require('blueimp-file-upload-expressjs-node-v4')(options);*/
    var storage = multer.diskStorage({
        destination: function (req, file, cb) {
            cb(null, path);
        },
        filename: function (req, file, cb) {
            cb(null, filename);
        }
    });
    var upload = multer({ storage: storage }).single("file[]");


    //uploader.post(function (err, obj) {
    upload(req, res, function (err) {

        if (err) {
            res.status(500).send(err);
        }
        else {
            res.sendStatus(200);
        }
    });
};

exports.delPackage = function(req, res, next){
    var username = (isMultiuser)?req.session.user:"singleuser";
    var pkgname = req.params.pkgname;
    var dirpath = module.parent.exports.set("dirname")+ "/" + getUserdirPath(req, isMultiuser)
                  + "_package/";

    if(fs.existsSync(dirpath + pkgname + ".def")) fs.unlinkSync(dirpath + pkgname + ".def");
    if(fs.existsSync(dirpath + pkgname + ".tlb")) fs.unlinkSync(dirpath + pkgname + ".tlb");
    res.sendStatus(200);
};

exports.existingPackage = function(req, res){
    var username = (isMultiuser)?req.session.user:"singleuser";
    var dirpath = module.parent.exports.set("dirname")+ "/" + getUserdirPath(req, isMultiuser) + "_package/";

    // Create path 'networks_package' if not exists
    if (!fs.existsSync(dirpath)) { fs.mkdirSync(dirpath); }

    var pkgs = {};
    var files = _.chain(fs.readdirSync(dirpath))
                 .filter(function(f){
                    return f.slice(-4) === ".tlb" || f.slice(-4) === ".def";})
                 .value();
    for(var i=0; i<files.length; i++){
        var parsed = files[i].split(".");
        console.log(parsed, pkgs, _.keys(pkgs));
        if(_.indexOf(_.keys(pkgs), parsed[0]) === -1){
            pkgs[parsed[0]] = [parsed[1]];
        } else {
            pkgs[parsed[0]].push(parsed[1]);
        }
    }

    res.json(pkgs);
};

exports.addFile = function (req, res, next) {
    var username = (isMultiuser)?req.session.user:"singleuser";
    var type = req.params.type;
    var filename = req.params.name;
    var size = req.params.size || -1;
    var root = fileSys.getRoot();
    var filePath = root + '/' + filename;

    // Check for existing file
    fs.stat(filePath, function (err, stats) {
/*        var options = {
            tmpDir: './tmp',
            uploadDir: root,
            uploadUrl:  '/file/' + type,
            storage : {
                type : 'local'
            },
            copyImgAsThumb : false
        };
        var uploader = require('blueimp-file-upload-expressjs-node-v4')(options);*/
        var storage = multer.diskStorage({
            destination: function (req, file, cb) {
                cb(null, root);
            },
            filename: function (req, file, cb) {
                cb(null, filename);
            }
        });
        var upload = multer({ storage: storage }).single("file[]");

        if (!err) { // File exist
            fs.renameSync(filePath, root + '/' + 'tmp_' + filename); // Creates temporary file
        }

        //uploader.post(req, res, function (err, obj) {
        upload(req, res, function (err) {
            if (err) { // Upload error
                res.status(500).send(err);
            } else {
                var validate = function (type, callback) {
                    if (type === 'zip') {
                        //validate if valid tf
                        exec('harktoolcli-validtf ' + '"' + filePath + '"', function (error, stdout, stderr) {
                            if (error !== null) {
                                //if error occurs, it means it is not a valid tf, therefore set to others
                                callback('others');
                            }
                            else {
                                //initialize type to tf
                                callback('tf');
                            }
                        });
                    }
                    else {
                        callback(type);
                    }
                };

                var uploadStats = fs.statSync(filePath);
                // Validates for type zip
                validate (type, function (data) {
                    type = data;
                    fileCache.setData(root, filename, {"filename": filename, "type": type, "size": uploadStats.size || 0, "path": filePath});
                });

                fs.stat(root + '/' + 'tmp_' + filename, function (err, stats) { // Check for temp files
                    if (!err) { // Deletes temp files
                        fs.unlinkSync(root + '/' + 'tmp_' + filename);
                    }
                    res.send({textStatus: 'success'}); // Sends response
                });
            }
        });
    });
};

//checks file type according to filename
function checkType(filename){
    var ext = filename.split(".").slice( -1 )[0];
    if (ext === "n"){
        return "network";
    }
    if (_.indexOf(["dat", "tff"], ext) !== -1){
        return "tf";
    }
    if (_.indexOf(["zip"], ext) !== -1){
        return "zip";
    }
    if (_.indexOf(["wav", "raw"], ext) !== -1){
        return "audio";
    }
    return "others";
}

// Commmented this since socket.io is not used
/*exports.initIO = function(_io) {
    io = _io;
    //connect to server
    io.sockets.on('connection', function (socket) {
        _socket = socket;
    });
};*/

function saveNetwork(name, content, req){
    // Includes directory when saving network
    fs.writeFileSync(name, content);
    fs.chmodSync(name, 0755);
}

function saveNetworkPerSheet(name, content, req){
    var path = process.env.FLOWDESIGNER_PATH + "/user-generated/";
    if(fs.existsSync(path)) {
        fs.writeFileSync(path + name, content);
        fs.chmodSync(path + name, 0755);
    }

}

function data2xmlPerSheet(network){
    var result = [];
    var params = [];

    // templates for xml conersion
    var tmplSheet   = '  <Network type="<%= type %>" name="<%= name %>">';
    var tmplNode    = '    <Node name="<%= id %>" type="<%= name %>" x="<%= pos[0] %>" y="<%= pos[1] %>">';
    var tmplNodeWithProcess = '    <Node name="<%= id %>" type="<%= name %>" x="<%= pos[0] %>" y="<%= pos[1] %>" process="<%= process %>">';
    var tmplNodeOut = '    <NetOutput name="<%= name %>" node="<%= nodeId %>" terminal="<%= termName %>" object_type="<%- object_type %>" description="<%- description %>"/>';
    var tmplNodeIn  = '    <NetInput name="<%= name %>" node="<%= nodeId %>" terminal="<%= termName %>" object_type="<%- object_type %>" description="<%- description %>"/>';
    var tmplNodeCond= '    <NetCondition name="<%= name %>" node="<%= nodeId %>" terminal="<%= termName %>"/>';
    var tmplParam   = '      <Parameter name="<%= name %>" type="<%- type %>" value="<% if(type==="object"){ %><%- value %><% } else { %><%= value %><% } %>" description="<%- description %>"/>';
    var tmplLink    = '    <Link from="<%= fromNodeId %>" output="<%= fromTermName %>" to="<%= toNodeId %>" input="<%= toTermName %>"/>';

    // headers
    result.push('#!/usr/bin/env batchflow');
    result.push('<?xml version="1.0"?>');
    result.push('<Document category="User-generated">');

    // generate for each sheets
    var nodesPerSheet = _.groupBy(network.nodes, "sheet");
    _.each(_.keys(nodesPerSheet), function(sheet) {
        result.push(_.template(tmplSheet)(network.sheets[sheet]));

        // generate nodes and their parameters
        _.each(_.filter(network.nodes, function(node){return node.sheet == sheet;}),
            function(node){
                if(_.has(node, "process")){
                    result.push(_.template(tmplNodeWithProcess)(node));
                }
                else{
                    result.push(_.template(tmplNode)(node));
                }
                _.chain(node.parameters)
                    .each(function(parameter){
                        parameter.description = parameter.description.replace(/[ \n\r]*$/g, "");
                        if(parameter.type == "subnet_param") {
                            params.push(parameter);
                        }
                        if(parameter.type === "string") {
                            parameter.value = parameter.value.replace(/</g, "&lt;").replace(/>/g, "&gt;");
                        }
                        result.push(_.template(tmplParam)(parameter));
                    });
                result.push("    </Node>");
            });

        // generate links
        _.each(_.filter(network.links, function(link){return link.sheet == sheet;}),
            function(link){result.push(_.template(tmplLink)(link));}
        );

        // generate network I/O/Condition
        _.each(_.filter(network.terminalRoles, function(role){return role.sheet == sheet;}),
            function(role){
                switch(role.role){
                    case "input"    : result.push(_.template(tmplNodeIn  )(role));break;
                    case "output"   : result.push(_.template(tmplNodeOut )(role));break;
                    case "condition": result.push(_.template(tmplNodeCond)(role));break;
                }
            }
        );
        result.push("  </Network>");
        _.each(params, function(p) {
            result.push(_.template(tmplParam)(p));
        });
    });

    // footers
    result.push("</Document>");
    result.push("");
    return result.join("\n");
}

function data2xml(network){
    var result = [];

    // templates for xml conersion
    var tmplSheet   = '  <Network type="<%= type %>" name="<%= name %>">';
    var tmplNode    = '    <Node name="<%= id %>" type="<%= name %>" x="<%= pos[0] %>" y="<%= pos[1] %>">';
    var tmplNodeWithProcess = '    <Node name="<%= id %>" type="<%= name %>" x="<%= pos[0] %>" y="<%= pos[1] %>" process="<%= process %>">';
    var tmplNodeOut = '    <NetOutput name="<%= name %>" node="<%= nodeId %>" terminal="<%= termName %>" object_type="<%- object_type %>" description="<%- description %>"/>';
    var tmplNodeIn  = '    <NetInput name="<%= name %>" node="<%= nodeId %>" terminal="<%= termName %>" object_type="<%- object_type %>" description="<%- description %>"/>';
    var tmplNodeCond= '    <NetCondition name="<%= name %>" node="<%= nodeId %>" terminal="<%= termName %>"/>';
    var tmplParam   = '      <Parameter name="<%= name %>" type="<%- type %>" value="<% if(type==="object"){ %><%- value %><% } else { %><%= value %><% } %>" description="<%- description %>"/>';
    var tmplLink    = '    <Link from="<%= fromNodeId %>" output="<%= fromTermName %>" to="<%= toNodeId %>" input="<%= toTermName %>"/>';
    var tmplNote    = '    <Note x="<%= pos[0] %>" y="<%= pos[1] %>" visible="<%= visible %>" text="<%= text %>" index="<%= index %>"/>';
    var tmplDefaultParam = '  <Parameter name="<%= name %>" type="<%- type %>" value="<% if(type==="object"){ %><%- value %><% } else { %><%= value %><% } %>"/>';

    // headers
    result.push('#!/usr/bin/env batchflow');
    result.push('<?xml version="1.0"?>');
    result.push("<Document>");

    // generate for each sheets
    var nodesPerSheet = _.allKeys(network.sheets);

    _.each(nodesPerSheet, function(sheet){
        result.push(_.template(tmplSheet)(network.sheets[sheet]));

        // generate nodes and their parameters
        _.each(_.filter(network.nodes, function(node){return node.sheet == sheet;}),
            function(node){
            if(_.has(node, "process")){
                result.push(_.template(tmplNodeWithProcess)(node));
            }
            else{
                result.push(_.template(tmplNode)(node));
            }
            _.chain(node.parameters)
             .each(function(parameter){
                parameter.description = parameter.description.replace(/[ \n\r]*$/g, "");
                if(parameter.type === "string") {
                    parameter.value = parameter.value.replace(/</g, "&lt;").replace(/>/g, "&gt;");
                }
                result.push(_.template(tmplParam)(parameter));
             });
            result.push("    </Node>");
        });

        // generate links
        _.each(_.filter(network.links, function(link){return link.sheet == sheet;}),
            function(link){result.push(_.template(tmplLink)(link));}
        );

        // generate network I/O/Condition
        _.each(_.filter(network.terminalRoles, function(role){return role.sheet == sheet;}),
            function(role){
                switch(role.role){
                    case "input"    : result.push(_.template(tmplNodeIn  )(role));break;
                    case "output"   : result.push(_.template(tmplNodeOut )(role));break;
                    case "condition": result.push(_.template(tmplNodeCond)(role));break;
                }
            }
        );

        // generate notes
        _.each(_.filter(network.notes, function(notes){return notes.sheet == sheet;}),
            function(notes){
                var text = notes.text;
                text = text.replace(/\r?\n/g, "&#10;");
                notes.text = text;
                result.push(_.template(tmplNote)(notes));
            }
        );
        result.push("  </Network>");
    });

    // generate default parameter
    _.each(network.defaultParams, function(parameter){
        if(parameter.type === "string") {
            parameter.value = parameter.value.replace(/</g, "&lt;").replace(/>/g, "&gt;");
        }
        result.push(_.template(tmplDefaultParam)(parameter));
    });

    // footers
    result.push("</Document>");
    result.push("");
    return result.join("\n");
}

function xml2dataPerSheet(xml){
    // console.log(util.inspect(xml, depth=10));
    var sheets = [];
    _.each(xml.Document.Network, function(network){
        // console.log(util.inspect(network, depth=10));
        var sheet = {name: network.$.name,
            type: network.$.type,
            links: [],
            nodes: [],
            roles: [],
            notes: []};

        // nodes
        _.each(network.Node, function(node){
            // console.log(util.inspect(node));
            var tmpnode = {
                id: node.$.name,
                index: parseInt(node.$.name.split("_").slice(-1), 10),
                name: node.$.type,
                pos: [Math.floor(parseInt(node.$.x, 10) / 10) * 10,
                    Math.floor(parseInt(node.$.y, 10) / 10) * 10],
                parameters: [],
                sheet: sheet.name
            };

            _.each(node.Parameter, function(param){
                // console.log(util.inspect(param));
                tmpnode.parameters.push({
                    name: param.$.name,
                    type: param.$.type,
                    list: [],
                    value: param.$.value,
                    description: param.$.description
                });
            });
            sheet.nodes.push(tmpnode);
        });

        // links
        _.each(network.Link, function(link){
            // console.log(util.inspect(link));
            var tmplink = {
                fromNodeId: link.$.from,
                fromTermName: link.$.output,
                toNodeId: link.$.to,
                toTermName: link.$.input,
                sheet: sheet.name
            };
            sheet.links.push(tmplink);
        });

        // terminalRoles
        _.each(network.NetInput, function(term){
            // console.log(term);
            sheet.roles.push({
                name       : term.$.name,
                nodeId     : term.$.node,
                description: term.$.description,
                object_type: term.$.object_type,
                role       : "input",
                sheet      : sheet.name,
                termName   : term.$.terminal
            });
        });

        _.each(network.NetOutput, function(term){
            // console.log(term);
            sheet.roles.push({
                name       : term.$.name,
                nodeId     : term.$.node,
                description: term.$.description,
                object_type: term.$.object_type,
                role       : "output",
                sheet      : sheet.name,
                termName   : term.$.terminal
            });
        });

        _.each(network.NetCondition, function(term){
            // console.log(term);
            sheet.roles.push({
                name       : term.$.name,
                nodeId     : term.$.node,
                role       : "condition",
                sheet      : sheet.name,
                termName   : term.$.terminal
            });
        });

        // Note
        _.each(network.Note, function(note){
            // console.log(util.inspect(note));
            var tmpnote = {
                pos: [Math.floor(parseInt(note.$.x, 10) / 10) * 10,
                    Math.floor(parseInt(note.$.y, 10) / 10) * 10],
                visible: parseInt(note.$.visible, 10),
                text: note.$.text,
                index: parseInt(note.$.index),
                noteId: "note" + parseInt(note.$.index),
                sheet: sheet.name
            };
            // skip invisible note because of comment
            if (tmpnote.visible !== 0) {
                sheet.notes.push(tmpnote);
            }
        });
        sheets.push(sheet);
    });

    _.each(xml.Document, function(document) {
        if(document.category !== undefined) {
            console.log("CATEGORY: ", document.category);
        }
    });

    // make inputs for dynamic nodes
    var dynamicNodeNames = _.chain(sheets)
        .pluck("name")
        .reject(function(n){return n === "MAIN";})
        .value();
    var dynamicNodes = _.chain(sheets)
        .pluck("nodes")
        .map(function(nodes){
            return _.filter(nodes, function(n){
                return dynamicNodeNames.indexOf(n.name) !== -1; });
        })
        .flatten()
        .value();
    _.each(dynamicNodes, function(dyn){
        var sheet = _.find(sheets, function(s){return s.name === dyn.name;});
        dyn.inputs = _.chain(sheet.roles)
            .filter(function(t){return t.role === "input";})
            .map(function(t){
                return {
                    description: t.description,
                    name       : t.name,
                    type       : t.object_type};
            })
            .value();
        dyn.outputs = _.chain(sheet.roles)
            .filter(function(t){return t.role === "output";})
            .map(function(t){
                return {
                    description: t.description,
                    name       : t.name,
                    type       : t.object_type};
            })
            .value();
    });

    var defaultParams = [];
    _.each(xml.Document.Parameter, function(param){
        console.log("Parameter="+param.$.name+","+param.$.type+","+param.$.value);
        var tmpDefaultParam = {
                    name: param.$.name,
                    type: param.$.type,
                    value: param.$.value
            };
        defaultParams.push(tmpDefaultParam);
    });

    // fix types
    var ret = adjustSheets(sheets);
    if(defaultParams.length) {
        ret.defaultParams = defaultParams;
    }
    return ret;
}

function xml2data(xml){
    // console.log(util.inspect(xml, depth=10));
    var sheets = [];
    _.each(xml.Document.Network, function(network){
        // console.log(util.inspect(network, depth=10));
        var sheet = {name: network.$.name,
                     type: network.$.type,
                     links: [],
                     nodes: [],
                     roles: [],
                     notes: []};

        // nodes
        _.each(network.Node, function(node){
            // console.log(util.inspect(node));
            var tmpnode = {
                id: node.$.name,
                index: parseInt(node.$.name.split("_").slice(-1), 10),
                name: node.$.type,
                pos: [Math.floor(parseInt(node.$.x, 10) / 10) * 10,
                      Math.floor(parseInt(node.$.y, 10) / 10) * 10],
                parameters: [],
                sheet: sheet.name
            };

            _.each(node.Parameter, function(param){
                // console.log(util.inspect(param));
                tmpnode.parameters.push({
                    name: param.$.name,
                    type: param.$.type,
                    list: [],
                    value: param.$.value,
                    description: param.$.description
                });
            });
            sheet.nodes.push(tmpnode);
        });

        // links
        _.each(network.Link, function(link){
            // console.log(util.inspect(link));
            var tmplink = {
                fromNodeId: link.$.from,
                fromTermName: link.$.output,
                toNodeId: link.$.to,
                toTermName: link.$.input,
                sheet: sheet.name
            };
            sheet.links.push(tmplink);
        });

        // terminalRoles
        _.each(network.NetInput, function(term){
            // console.log(term);
            sheet.roles.push({
                name       : term.$.name,
                nodeId     : term.$.node,
                description: term.$.description,
                object_type: term.$.object_type,
                role       : "input",
                sheet      : sheet.name,
                termName   : term.$.terminal
            });
        });

        _.each(network.NetOutput, function(term){
            // console.log(term);
            sheet.roles.push({
                name       : term.$.name,
                nodeId     : term.$.node,
                description: term.$.description,
                object_type: term.$.object_type,
                role       : "output",
                sheet      : sheet.name,
                termName   : term.$.terminal
            });
        });

        _.each(network.NetCondition, function(term){
            // console.log(term);
            sheet.roles.push({
                name       : term.$.name,
                nodeId     : term.$.node,
                role       : "condition",
                sheet      : sheet.name,
                termName   : term.$.terminal
            });
        });

        // Note
        _.each(network.Note, function(note){
            // console.log(util.inspect(note));
            var tmpnote = {
                pos: [Math.floor(parseInt(note.$.x, 10) / 10) * 10,
                    Math.floor(parseInt(note.$.y, 10) / 10) * 10],
                visible: parseInt(note.$.visible, 10),
                text: note.$.text,
                index: parseInt(note.$.index),
                noteId: "note" + parseInt(note.$.index),
                sheet: sheet.name
            };
            // skip invisible note because of comment
            if (tmpnote.visible !== 0) {
                sheet.notes.push(tmpnote);
            }
        });
        sheets.push(sheet);
    });

    // make inputs for dynamic nodes
    var dynamicNodeNames = _.chain(sheets)
                            .pluck("name")
                            .reject(function(n){return n === "MAIN";})
                            .value();
    var dynamicNodes = _.chain(sheets)
                        .pluck("nodes")
                        .map(function(nodes){
                            return _.filter(nodes, function(n){
                                return dynamicNodeNames.indexOf(n.name) !== -1; });
                        })
                        .flatten()
                        .value();
    _.each(dynamicNodes, function(dyn){
        var sheet = _.find(sheets, function(s){return s.name === dyn.name;});
        dyn.inputs = _.chain(sheet.roles)
                      .filter(function(t){return t.role === "input";})
                      .map(function(t){
                        return {
                            description: t.description,
                            name       : t.name,
                            type       : t.object_type};
                      })
                      .value();
        dyn.outputs = _.chain(sheet.roles)
                      .filter(function(t){return t.role === "output";})
                      .map(function(t){
                        return {
                            description: t.description,
                            name       : t.name,
                            type       : t.object_type};
                      })
                      .value();
    });

    //fix types
    return adjustSheets(sheets);
}

function adjustSheets(sheets) {
    var network = {};
    network.sheets = {};
    _.each(sheets, function(s){network.sheets[s.name] = {name: s.name, type: s.type};});
    network.nodes  = _.chain(sheets).pluck("nodes").flatten().value();
    network.links  = _.chain(sheets).pluck("links").flatten().value();
    network.terminalRoles = _.chain(sheets).pluck("roles").flatten().value();
    network.notes  = _.chain(sheets).pluck("notes").flatten().value();

    var nodesNotesArray = _.union(network.nodes, network.notes);
    // adjust pos for each sheet
    // 1. shift
    _.each(_.keys(network.sheets), function(sheet){
        var common = _.chain(nodesNotesArray)
            .filter(function(n){return n.sheet === sheet;})
            .pluck("pos").value();
        var minx = _.chain(common).map(function(p){return p[0];}).min().value();
        var miny = _.chain(common).map(function(p){return p[1];}).min().value();
        _.chain(nodesNotesArray)
            .filter(function(n){return n.sheet === sheet;})
            .each(function(n){
                n.pos[0] -= minx;
                n.pos[1] -= miny;
            });
    });
    // 2. change scale
    var size = [6000, 6000]; // all node positions must be in this size
    _.each(network.nodes, function(n){
        //add margin
        n.pos[0] += 100;
        n.pos[1] += 100;

        if(n.pos[0] > size[0]) n.pos[0] = size[0];
        if(n.pos[1] > size[1]) n.pos[1] = size[1];
    });
    _.each(network.notes, function(n){
        //add margin
        n.pos[0] += 100;
        n.pos[1] += 100;

        if(n.pos[0] > size[0]) n.pos[0] = size[0];
        if(n.pos[1] > size[1]) n.pos[1] = size[1];
    });

    // re-index all nodes
    if(network.nodes.length > 1) {
        // build new id table.
        var tmp = "___tmp___";
        var counter = {};
        var idtable = {};
        _.each(network.nodes, function(n){
            if(typeof counter[n.name] === "undefined")
                counter[n.name] = 1;
            else
                counter[n.name] += 1;
            idtable[n.sheet + tmp + n.id]
                = "node_" + n.name + "_" + counter[n.name];
        });

        // rename all nodes
        _.each(network.nodes, function(n){
            n.id = idtable[n.sheet + tmp + n.id] || n.id;
        });

        // rename all links
        _.each(network.links, function(l){
            l.fromNodeId = idtable[l.sheet + tmp + l.fromNodeId] || l.fromNodeId;
            l.toNodeId   = idtable[l.sheet + tmp + l.toNodeId] || l.toNodeId;
        });

        // rename all terminalRoles
        _.each(network.terminalRoles, function(r){
            r.nodeId = idtable[r.sheet + tmp + r.nodeId] || r.nodeId;
        });

        // console.log(network.sheets);
    }

    return network;
}

function lock(){
    var lock = path.join(process.cwd(), "lock");
    if(!fs.exists(lock)){
        fs.writeFileSync(lock, "");
        return;
    }

    while(fs.exists(lock)){
        console.log("waiting");
    }
}

function unlock(){
    var lock = path.join(process.cwd(), "lock");
    fs.unlinkSync(lock);
}


function getUserdirPath(req, isMultiuser){
    if(isMultiuser){
        return "userdata/" + req.session.user;
    } else{
        return "userdata/networks";
    }
}


function checkData(obj, type){
    if(type === "network"){
        if(typeof(obj) !== 'string') return "This is not a text file";
        if(obj.indexOf("#!") !== 0) return "This does not starts with #!";
        if(obj.indexOf("<?xml") === -1) return "This not an xml file";
        if(obj.indexOf("batchflow") === -1) return "No word 'bathcflow' found";
        return "";
    }
    else{
        return "unknown data type";
    }
    return "This should not happen";
}

// Module for file tree
var fileSys = (function () {
    var root = "";

    var setRoot = function (dir) {
        if (!_.isUndefined(dir) && (dir !== 'undefined')) {
            root = ((distribution !== 'Windows') ? '/' : '') + cleanpath(dir);
        } else {
            root = getDefaultDir();
        }
    };

    var getRoot = function () {
        return root;
    };

    var getDefaultDir = function (req) {
        var reg = new RegExp(path.delimiter, 'g');
        
        if (distribution !== 'Windows') {
            return "/" + cleanpath(process.cwd().replace(reg, "/") + "/" + getUserdirPath(req, isMultiuser));
        }
        else {
            return cleanpath(process.cwd().replace(reg, "/") + "/" + getUserdirPath(req, isMultiuser));
        }
    };

    var resolve = function (checkPath) {
        if (distribution !== 'Windows') {
            return path.resolve(checkPath);
        } else {
            // Consistant path.resolve for Windows
            if (checkPath.match(/^[a-zA-Z]:$/)) {
                // Prevent c: -> cwd
                checkPath += path.sep;
            }
            checkPath = path.resolve(checkPath);
            if (checkPath.match(/^[a-zA-Z]:[\/\\]$/)) {
                // c:\ -> c:
                checkPath = checkPath.slice(0, -1);
            }
            return checkPath;
        }
    };

    var cleanpath = function (checkPath) {
        checkPath = checkPath.replace(/\/|\\/g, '|');
        checkPath = _.without(checkPath.split('|'), "");
        checkPath = checkPath.join('/');
        return checkPath.trim();
    };

    var real = function (checkPath) {
        return fileSys.resolve(checkPath);
    };

    var processPath = function (dir, id) {
        id = id.replace('/', path.sep);
        id = id.trim(path.sep);
        id = real(path.join(dir, id));
        return id;
    };

    var processId = function (basedir, checkPath) {
        checkPath = real(checkPath);
        checkPath = checkPath.substr(basedir.length);
        checkPath = checkPath.replace('/', path.sep);
        checkPath = checkPath.trim(path.sep);
        return checkPath.length ? checkPath : '/';
    };

    var list = function (basedir, id, with_root, callback) {
        with_root = (!_.isUndefined(with_root) && with_root !== 'undefined') ? with_root : false;

        basedir = fileSys.resolve((basedir) ? basedir : path.parse(process.cwd()).root);

        var dir = processPath(basedir, id);
        try {
/*            var list = fs.readdirSync(dir);

            var listHidden = function (dir, distribution, callback) {
                if (distribution === 'Windows') {
                    exec('dir '+ dir +' /b /A:DH', function (err, stdout, stderr) {
                        if (!err) {
                            stdout = stdout.toString().split('\r\n');
                            callback (stdout);
                        }
                        else {
                            callback([]);
                        }
                    });
                }
                else {
                    exec('find '+ dir +' -type f -iname ".*" -print', function (err, stdout, stderr) {
                        if (!err) {
                            var temp = [];

                            stdout = stdout.trim().split('\n');

                            _.each(stdout, function (val) {
                                temp.push(val.replace(dir + '/',''));
                            });

                            callback (temp);
                        }
                        else {
                            callback([]);
                        }
                    });
                }
            };*/

            if (fs.statSync(dir + path.sep).isDirectory()) {
                //listHidden(dir, distribution,function (hiddenList) {
                    var res = [];
                    //list = _.difference(list, hiddenList);

                    var list = [];
                    // NOTE:
                    // Windowsのfind(str)、Linuxのgrepともにマッチした行が無かったらcode=1を返す。
                    // fs.exec(Sync)ではcode>0の場合、例外をThrowするため、後処理でcode=0になるようworkaroudを入れている。
                    // node.js v4.x以降の場合、以下の様に簡潔に書ける。
                    // spawnSync('ls "'+ dir +'" -F | grep .*/$', [], {"shell":true});
                    var childDir;
                    if (distribution === 'Windows') {
                        // List all directories, because c:\ProgramData is hidden
                        childDir = execSync('dir "'+ dir +'\\" /ad /b');
                        if (childDir) {
                            list = childDir.toString().split('\r\n');
                        }
                    } else {
                        childDir = execSync('ls "'+ dir +'/" -F | grep .*[/@]$; GREPCODE=$?; [ $GREPCODE -le 1 ] && exit 0 || exit $GREPCODE');
                        if (childDir) {
                            list = childDir.toString().replace(/[\/@]/g, "").split('\n');
                        }
                    }
                    list = _.compact(list);

                    _.each(list, function (item) {
                        if (item == '.' || item == '..' || item === null) { return true; }
                        var tmp = item.match('([^ a-zа-я-_0-9.]+)ui');
                        if (tmp === false || tmp === 1) { return true; }
                        var dirCheck = dir + path.sep + item;
                        var regex = new RegExp('/(^%?[\S?\s?]*%?$)/');

                        try {
                            if (fs.statSync(dirCheck).isDirectory() && !regex.test(item)) {
                                //try {
                                    var cmd;
                                    if (distribution === 'Windows') {
                                        cmd = 'dir "' + dirCheck + '\\" /ad /b | find /c /v "" & if %ERRORLEVEL% == 1 ( cd )';
                                    } else {
                                        cmd = 'ls "' + dirCheck + '/" -F | grep .*[/@]$ -c; GREPCODE=$?; [ $GREPCODE -le 1 ] && exit 0 || exit $GREPCODE';
                                    }
                                    //var listC = fs.readdirSync(dirCheck);
                                    //if (listC.length <= 0) {
                                    if (execSync(cmd).toString() === 0) {
                                        res.push({text: item, children: false, id: cleanpath(processId(basedir, path.join(dir, item))), icon: 'folder'});
                                    }
                                    else {
                                        res.push({text: item, children: true, id: cleanpath(processId(basedir, path.join(dir, item))), icon: 'folder'});
                                    }
                                //}
                                //catch (err) {
                                //    return true;
                                //}
                            }
                        }
                        catch (err) {
                            return true;
                        }
                    });

                    if (with_root && (processId(basedir, dir) === '/' || processId(basedir, dir) === '\\')) {
                        res = {text: cleanpath(path.basename(basedir)) || basedir, children: res, id: '/', icon: 'folder', state: {opened: true, disabled: false}};
                    }

                    callback(res);
                //});
            }
        }
        catch (err) {
            callback({err: 'Could not list path: ' + dir});
        }
    };

    return {
        list: list,
        setRoot: setRoot,
        getRoot: getRoot,
        getDefaultDir: getDefaultDir,
        cleanpath: cleanpath,
        resolve: resolve,
    };
})();

exports.getRoot = function(){return fileSys.getRoot();};

var fileCache = (function () {
    var storePath = path.join(process.cwd(), "fileinfo-cache");
    var ext = ".json";
    var crypto = require("crypto");
    var store = function() {
        return storePath;
    };
    var hash = function(str) {
        return crypto.createHash("md5").update(str || "").digest("hex");
    };
    var cacheFileName = function(dirPath, fileName) {
        return hash(dirPath) + "_" + fileName + ext;
    };
    var getData = function(dirPath, fileName, modifiedAt) {
        var ret;
        var fd;
        try {
            fd = fs.openSync(path.join(store(), cacheFileName(dirPath, fileName)), 'r');
            var stats = fs.fstatSync(fd);
            if (!modifiedAt || modifiedAt.getTime() <= stats.ctime.getTime()) {
                var buf = new Buffer(stats.size);
                fs.readSync(fd, buf, 0, stats.size, 0);
                ret = JSON.parse(buf.toString());
            }
            fs.closeSync(fd);
        } catch (e) {
            if (e.code !== "ENOENT") {
                console.log("Exception occuerd!: ", e);
            }
            try {
                if (fd) {
                    fs.closeSync(fd);
                }
            } catch (e) {
                console.log("Exception occuerd!: ", e);
            }
        }
        return ret;
    };
    var setData = function(dirPath, fileName, data) {
        try {
            var oldmask = process.umask(0);
            fs.writeFileSync(path.join(store(), cacheFileName(dirPath, fileName)), JSON.stringify(data), { mode: '0666', encoding: 'utf-8' });
            process.umask(oldmask);
        } catch (e) {
            console.log("Exception occuerd!: ", e);
        }
    };
    var deleteData = function(dirPath, fileName) {
        try {
            return fs.unlinkSync(path.join(store(), cacheFileName(dirPath, fileName)));
        } catch (e) {
            if (e.code !== "ENOENT") {
                console.log("Exception occuerd!: ", e);
            }
        }
    };
    var listCache = function(dirPath) {
        var list = [];
        var pref = hash(dirPath) + "_";
        try {
            var cacheFiles = fs.readdirSync(store());
            cacheFiles.filter(function(file) {
                return fs.statSync(path.join(store(), file)).isFile() && path.extname(file) === ext && file.lastIndexOf(pref, 0) === 0;
            }).forEach(function(file) {
                list.push(file.slice(pref.length, -(ext.length)));
            });
        } catch (e) {
            console.log("Exception occuerd!: ", e);
        }
        return list;
    };
    return {
        store: store,
        getData: getData,
        setData: setData,
        deleteData: deleteData,
        listCache: listCache,
    };
})();
